//Copyright (C) 2011, FIE LLC.

//See LICENSE.TXT for details.

using System;
using libsc;
using System.Collections.Generic;
namespace sc
{
	class MainClass
	{
		public static void Main (string[] args)
		{
            System.Console.WriteLine("sc!");
            try
            {
                libsc.SysConfig.LoadPlugins();
            }
            catch (libsc.BadEnvironmentException e)
            {
                System.Console.Error.WriteLine("Environment Error: " + e.Message);
                System.Environment.Exit(-1);
            }
            ContextLaunchingPlatform plat = SysConfig.NewContextLaunchingPlatform();
            //ok... here we go.
            //this command is often called by a SubProcess context launcher when it runs the
            //next context.  So it has run itself and it has set its "next context" up
            //it has adjusted this environment for this incoming context.

            //the other possibility is that we have been called outside a context.
            //if so, then we expect to find a path to a serialized context
            //in our args.

            //so we get the current context.
            //from there, we get the "next context" form its settings.
            //then we launch it.
            System.IO.MemoryStream ms;
            Context parentContext = null;
            if (CurrentContext.HasContext())
            {
                System.Console.WriteLine("in context");
                parentContext = CurrentContext.GetCurrentContext();
                ms = new System.IO.MemoryStream(parentContext.GetCurrentSettings().NextSettings);
            }
            else
            {
                System.Console.WriteLine("not in context");
                if (args.Length < 1){
                    System.Console.Error.WriteLine("We are not in a context and we've not been passed a file on the commandline.");
                    System.Environment.Exit(-1);
                }
                string path = args[0];
                if (!System.IO.File.Exists(path)){
                    System.Console.Error.WriteLine("The passed file does not exist.");
                    System.Environment.Exit(-1);
                }
                ms = new System.IO.MemoryStream(System.IO.File.ReadAllBytes(path));
            }
            System.Runtime.Serialization.Formatters.Binary.BinaryFormatter bf = new System.Runtime.Serialization.Formatters.Binary.BinaryFormatter();
            object o = bf.Deserialize(ms);
            if (!(o is ContextSettings))
            {
                System.Console.Error.WriteLine("While the given data does deserialize, its not a ContextSettings object.  Something is very wrong.");
                System.Environment.Exit(-1);
            }
            ContextSettings cs = o as ContextSettings;
            ContextLauncher launcher;
            if (!plat.GetContextLaucher(cs.NameSpace, cs.Name, out launcher))
            {
                System.Console.Error.WriteLine(string.Format("The context referenced in the settings does not exist in this context: %s/%s",cs.NameSpace,cs.Name));
                System.Environment.Exit(-1);
            }
            string reason;
            if (!launcher.CanRun(out reason,parentContext)){
                System.Console.Error.WriteLine(string.Format("The given context cannot run.  It says: " + reason));
                System.Environment.Exit(-1);
            }
            launcher.LaunchContext(cs);
		}
	}
}

